module asr_envelope (
            input clk,
            input chipselect,
            input write,
            input [15:0] address,
            input [15:0] writedata,
            input logic signed [15:0] sample_in,
            input pressed,
            output logic [15:0] sample_out
        );

logic signed [15:0] amplitude;
typedef enum logic [2:0] {IDLE, ATTACK, SUSTAIN, RELEASE} statetype;

statetype next_state;
statetype present_state = IDLE;

logic [7:0] in_attack  = 8'd64;
logic [7:0] in_release = 8'd32;

always_ff @(posedge clk) begin
    if (chipselect && write) begin
        // if address is 0x8800 then
        // writedata contains attack/release values
        if (address == 16'h8800) begin
            in_attack  <= writedata[7:0];
            in_release <= writedata[15:8];
        end
    end
end

reg [30:0] ctr_attack;
always @(posedge clk) begin
    if (ctr_attack == 24'd5000) begin
        ctr_attack <= 24'd0;
    end
    else ctr_attack <= ctr_attack + 31'd1;
end


always_ff @(posedge clk) begin
    //if(!pressed) present_state <= IDLE;
    //else present_state <= next_state;
    present_state <= next_state;
end// adsr_envelope state 

// State Transitions
always_comb begin 
    case(present_state)
        IDLE : if(pressed) next_state = ATTACK;
        else next_state = IDLE;

        ATTACK: begin
            if(amplitude >= 16'd31743) next_state = SUSTAIN;
            else if (!pressed) next_state = RELEASE;
        else next_state = ATTACK;
        end

        SUSTAIN: if(!pressed) next_state = RELEASE;
        else next_state = SUSTAIN;

        RELEASE: begin
        if (amplitude <= 16'd1024 ) 
            next_state = IDLE;
        else if (amplitude > 16'd0 && pressed)
            next_state = ATTACK;
        else next_state = RELEASE;
        end
        default:
            next_state = IDLE;
endcase // present_state
end

// Output definitions
always_ff @(posedge clk ) begin 
    case(present_state)
        IDLE :  amplitude <= 16'd0;
        
        ATTACK: if (ctr_attack == 24'd1000) amplitude <= amplitude + in_attack;

        SUSTAIN: amplitude <= 16'd31743;

        RELEASE: begin
                    if (ctr_attack == 24'd1000) amplitude <= amplitude - in_release;
                    else if (amplitude <= 16'd1024 ) amplitude <= 16'd0;
         end
    endcase // present_state
end

logic signed [31:0] temp_mult;

always_comb begin : proc_sample_in
    temp_mult = (amplitude * sample_in);
    sample_out  = temp_mult[31:16];
end

endmodule
