/*
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <pthread.h>
#include <unistd.h>
#include "usbkeyboard.h"

//#define SERVER_HOST "0.0.0.0"
#define SERVER_HOST "127.0.0.1"
#define SERVER_PORT 42000

#define BUFFER_SIZE 16

#define USE_SOCKET


int sockfd; /* Socket file descriptor */

struct libusb_device_handle *keyboard;
uint8_t endpoint_address;

pthread_t network_thread;
void *network_thread_f(void *);


struct libusb_device_handle *keyboard;
uint8_t endpoint_address;

void onKeyChange(struct usb_keyboard_packet packet){
    int key_sig = packet.keycode[0];

    // -2A cuz we only support G2 to C#5 = 30 keys
    // 30 < 2^5 which nicely fits in a single 16bit wide bus
    // To support polyphony, we pass 5 bits of each keypress in a 
    // single 16bit-wide writedata
    // let's say we press G2,A3,A#3. Our writedata will be
    // 00001 00010 00011
    // lets say we press C#5, C5, B5. Our Writedata will be
    // 11111 11110 11101
    // BTW, in MIDI, G2 = d43 = 0x2B
    int key_val = packet.keycode[1] - 0x2A;
    //printf("%02x %02x\n", key_sig, key_val);

    // below algorithm leads to stuck keys
    // when more than 2 keys are released at the same time
    static int n_keypress;
    static int keys_pressed[3];

    // Key press
    if (key_sig == 0x90)
        if(n_keypress <3){
            for (int i=0; i < 3;i++){
                if (keys_pressed[i] == 0){
                    keys_pressed[i] = key_val;
                    n_keypress++;
                    break;}
            }
        }

    // Key release
    if (key_sig == 0x80)
        for (int i = 0; i < 3; i++)    
            if (keys_pressed[i] == key_val)
            {
                keys_pressed[i] = 0;
                n_keypress--;
            }
    
    //printf("Current array: %02x %02x %02x\n", keys_pressed[0], keys_pressed[1], keys_pressed[2]);
    //printf("n_keypress: %d\t", n_keypress);

    char buffer[BUFFER_SIZE];

    //sprintf(buffer, "%02x,%02x,%02x", keys_pressed[0], keys_pressed[1], keys_pressed[2]); 
    sprintf(buffer, "%02x,%02x,%02x", keys_pressed[0], keys_pressed[1], keys_pressed[2]); 
#ifdef USE_SOCKET
    if (key_val <= 0x1f && key_val >= 0x01){
        int n = write(sockfd, buffer, BUFFER_SIZE);
        if (n != BUFFER_SIZE)
            fprintf(stderr, "Error: write() failed. Is the server running?\n");

        printf("\n***\nTx size: %d\nTx data:%s\n***\n", n, buffer);
    }
    else 
        printf("\nVALID RANGE G2-C#5\n");
#endif
 }

int main()
{

  
  struct usb_keyboard_packet packet;
  int transferred;
  //char keystate[12];
  /* Open the keyboard */

  if ( (keyboard = openkeyboard(&endpoint_address)) == NULL ) {
    fprintf(stderr, "Did not find a keyboard\n");
    exit(1);
  }
 #ifdef USE_SOCKET

  struct sockaddr_in serv_addr;
  /* Create a TCP communications socket */
  if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 ) {
    fprintf(stderr, "Error: Could not create socket\n");
    exit(1);
  }

 
  /* Get the server address */
  memset(&serv_addr, 0, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = htons(SERVER_PORT);
  if ( inet_pton(AF_INET, SERVER_HOST, &serv_addr.sin_addr) <= 0) {
    fprintf(stderr, "Error: Could not convert host IP \"%s\"\n", SERVER_HOST);
    exit(1);
  }

  /* Connect the socket to the server */
  if ( connect(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
    fprintf(stderr, "Error: connect() failed.  Is the server running?\n");
    exit(1);
  }
#endif

// Dont need network thread, since i only wanna send to server?
  

  /* Look for and handle keypresses */
  for (;;) {
    libusb_bulk_transfer(keyboard, endpoint_address,
			      (unsigned char *) &packet, sizeof(packet),
			      &transferred, 0);
    //if (transferred == sizeof(packet)) {
   //   sprintf(keystate, " %02x %02x %02x %02x %02x %02x %02x %02x", packet.modifiers, packet.keycode[0],
   //           packet.keycode[1], packet.keycode[2], packet.keycode[3], packet.keycode[4],packet.keycode[5], packet.keycode[6]);
   //   printf("%s\n", keystate);
      onKeyChange(packet);
    
    //}
  }

  return 0;
}
